using System;
using System.Drawing;

namespace JALInterfaces
{
	// Refactored Drawable Interface
	public interface IDrawable
	{
		void DrawYourself(Graphics g);
	}
	// ASSERT P.X>=0 and P.Y>=0;
	public interface IPositional
	{
		Point Position
		{
			get;
			set;
		}
	}
	public interface ISizeable
	{
		int Size
		{
			get;
			set;
		}
	}
	public interface IColorable 
	{
		Color Color 
		{
			get;
			set;
		}
	}
	public interface IBrushWidth 
	{
		int BrushWidth
		{
			get;
			set;
		}
	}
	// Extensable Composite Interface IDrawableShape
	public interface IDrawableShape : IDrawable, IPositional, IColorable, ISizeable, IBrushWidth {}
	
	// Factor repeated code into Abstract Class with Default Implementations
	// Concrete classes of IDrawableShape should inherit from this class
	// and override DrawYourself
	public abstract class AbstractDrawableShape : IDrawableShape 
	{
		private int size=1;
		private Point position= new Point(0,0);
		private Color color= Color.Black;
		private int brushWidth= 1;
		public int BrushWidth 
		{
			get {return brushWidth;}
			set 
			{
				if (value>0) {brushWidth= value;}
			}
		}
		public int Size
		{
			get {return size;}
			set 
			{
				if (value>0) {size= value;}
			}
		}
		// ASSERT p.x and p.y >=0
		public Point Position
		{
			get {return position;}
			set 
			{
				if ((value.X >=0) && (value.Y >=0))
				{
					position= value;
				}
				else 
				{
					throw new ArgumentException();
				}
			}
		}
		public Color Color
		{
			get{return color;}
			set 
			{
				color= value;
			}
		}
		virtual public void DrawYourself(Graphics g)
		{
			g.DrawString("Drawable",new Font("Arial",size),new SolidBrush(color),position.X, position.Y,StringFormat.GenericDefault);
		}
	}
}